#!/usr/bin/env python
# @Project ：django_base_ai
# @File    : message_center.py
# @Author  : cx
# @Time    : 11/2/2025
# @Desc    : 消息中心创建、推送与 WebSocket 推送。
import json
import logging
import math
import time
from datetime import datetime

from asgiref.sync import async_to_sync
from channels.layers import get_channel_layer
from django.core.cache import cache
from django.db.models import Q
from django.http import StreamingHttpResponse
from django.shortcuts import render
from django.views.decorators.csrf import csrf_exempt
from django_celery_beat.models import CrontabSchedule, PeriodicTask
from django_restql.fields import DynamicSerializerMethodField
from rest_framework.decorators import action
from rest_framework.permissions import IsAuthenticated

from django_base_ai.settings import TASK_STATUS
from django_base_ai.system.models import MessageCenter, MessageCenterTargetUser
from django_base_ai.utils.json_response import DetailResponse, ErrorResponse, SuccessResponse
from django_base_ai.utils.permission import OpenApiPermission
from django_base_ai.utils.serializers import CustomModelSerializer
from django_base_ai.utils.viewset import CustomModelViewSet
from django_base_ai.websocket.websocket_config import websocket_push

logger = logging.getLogger(__name__)


class MessageCenterSerializer(CustomModelSerializer):
    """
    消息中心-序列化器
    """

    progress = DynamicSerializerMethodField()
    task_status = DynamicSerializerMethodField()
    cron = DynamicSerializerMethodField()

    def get_cron(self, instance, parsed_query):
        info = []
        periodic_tasks = instance.periodic_task.all()
        for p_task in periodic_tasks:
            info.append(
                {
                    "id": p_task.id,
                    "info": [
                        p_task.crontab.minute,
                        p_task.crontab.hour,
                        p_task.crontab.day_of_month,
                        p_task.crontab.month_of_year,
                        p_task.crontab.day_of_week,
                    ],
                }
            )
        return info

    def get_progress(self, instance, parsed_query):
        is_read_count = MessageCenterTargetUser.objects.filter(messagecenter=instance, is_read=True).count()
        total = MessageCenterTargetUser.objects.filter(messagecenter=instance).count()
        return math.floor((is_read_count / total) * 100) if total != 0 else 0

    def get_task_status(self, instance, parsed_query):
        if not instance.task_id:
            return "未发送"
        try:
            from celery.result import AsyncResult

            task = AsyncResult(instance.task_id)
            return TASK_STATUS.get(task.status, f"未知{task.status}") if task else "未找到"
        except Exception as e:
            print(e.args)
            return "未知"

    class Meta:
        model = MessageCenter
        fields = "__all__"
        read_only_fields = ["id"]


def ws_send_message(message_center_target_user_result, data, creator_name=""):
    targetuser_data = []
    create_datetime = data.create_datetime.strftime("%Y-%m-%d %H:%M:%S")
    for item in message_center_target_user_result:
        user_id = item.users.id
        targetuser_data.append(user_id)
        room_name = f"{user_id}_{cache.get(user_id, 0)}"
        websocket_push(
            room_name,
            message={
                "id": data.id,
                "msgType": "selfMessage",
                "title": data.title,
                "short_content": data.short_content,
                "thumbnail": data.thumbnail,
                "message_type": data.message_type,
                "content": data.content,
                "url": data.url,
                "target_type": data.target_type,
                "is_withdraw": data.is_withdraw,
                "ext_kwargs": data.ext_kwargs,
                "creator_name": creator_name,
                "create_datetime": create_datetime,
            },
        )
    return targetuser_data


class MessageCenterCreateUpdateSerializer(CustomModelSerializer):
    """
    消息中心-新增-序列化器
    """

    def save(self, **kwargs):
        # "* * * * *" => "{0}min {1}hour {2}day {3}month {4}week"
        instance = super().save(**kwargs)
        current_user = self.request.user
        upadate_crontab_ids = []
        cron_info = self.initial_data.get("cron", [])
        title = self.initial_data.get("title", "")
        if len(cron_info) < 1:
            return instance
        for c_info in cron_info:
            crontab_id = c_info.get("id", 0)
            cron_list = c_info.get("info", [])
            if len(cron_list) != 5:
                return instance
            if crontab_id == 0:
                # 添加任务定时器
                crontab = CrontabSchedule.objects.create(
                    minute=cron_list[0],
                    hour=cron_list[1],
                    day_of_month=cron_list[2],
                    month_of_year=cron_list[3],
                    day_of_week=cron_list[4],
                )
                periodic_task = PeriodicTask.objects.create(
                    crontab=crontab,
                    name=f"{time.time_ns()}-{title}",
                    enabled=False,
                    task="celery_tasks.tasks.send_message",
                    expires=instance.expires,
                    description=f"{current_user.username}创建",
                )
                periodic_task.args = [instance.id, current_user.id, current_user.name]
                periodic_task.save()
                upadate_crontab_ids.append(periodic_task.id)
            else:
                # 更新任务定时器
                CrontabSchedule.objects.filter(id=crontab_id).update(
                    minute=cron_list[0],
                    hour=cron_list[1],
                    day_of_month=cron_list[2],
                    month_of_year=cron_list[3],
                    day_of_week=cron_list[4],
                )
                upadate_crontab_ids.append(crontab_id)
        current_periodic_task = instance.periodic_task.all()
        current_crontab_ids = [item.id for item in current_periodic_task]
        if current_crontab_ids == upadate_crontab_ids:
            return instance
        del_ids = set(current_crontab_ids).difference(set(upadate_crontab_ids))
        if len(del_ids):
            CrontabSchedule.objects.filter(id__in=del_ids).delete()

        instance.periodic_task.set(upadate_crontab_ids)
        return instance

    class Meta:
        model = MessageCenter
        fields = "__all__"
        read_only_fields = ["id"]


def websocket_to_message(u_info, new_message_center, username):
    longth = len(u_info)
    index = 1
    for item in u_info:
        uid = item.users.id
        is_online = cache.get(uid, 0)
        room_name = f"{uid}_{is_online}"
        print(f"room_name={room_name}")
        websocket_push(
            room_name,
            message={
                "id": item.id,
                "msgType": "selfMessage",
                "title": new_message_center.title,
                "short_content": new_message_center.short_content,
                "thumbnail": new_message_center.thumbnail,
                "message_type": new_message_center.message_type,
                "content": new_message_center.content,
                "url": new_message_center.url,
                "target_type": new_message_center.target_type,
                "is_withdraw": new_message_center.is_withdraw,
                "ext_kwargs": new_message_center.ext_kwargs,
                "creator_name": username,
                "create_datetime": new_message_center.create_datetime.strftime("%Y-%m-%d %H:%M:%S"),
            },
        )
        chats = {
            "progress": 0 if longth == 0 or index == 0 else round((index / longth) * 100, 1),
            "is_end": False if index < longth else True,
        }
        yield f"data: {json.dumps(chats)}\n\n"
        index += 1


class MessageCenterViewSet(CustomModelViewSet):
    """
    消息中心接口
    list:查询
    create:新增
    update:修改
    retrieve:单例
    destroy:删除
    """

    queryset = MessageCenter.objects.order_by("-id")
    serializer_class = MessageCenterSerializer
    create_serializer_class = MessageCenterCreateUpdateSerializer
    update_serializer_class = MessageCenterCreateUpdateSerializer
    search_fields = ["title"]
    # extra_filter_backends = []

    @action(methods=["GET"], detail=False, permission_classes=[IsAuthenticated])
    def get_unread_msg(self, request):
        """获取未读消息数量"""
        self_user_id = self.request.user.id
        count = MessageCenterTargetUser.objects.filter(users__id=self_user_id, is_read=False).count()
        return DetailResponse(data={"count": count}, msg="获取成功")

    @action(methods=["POST"], detail=False, permission_classes=[IsAuthenticated])
    def send_ws_msg(self, request):

        channel_layer = get_channel_layer()

        room_name = f"user_{request.user.id}"
        send_data = request.data
        obj = {"type": "push.message", "json": send_data}
        async_to_sync(channel_layer.group_send)(room_name, obj)
        return DetailResponse()

    @action(methods=["POST"], detail=False, permission_classes=[IsAuthenticated])
    def read_one_click(self, request):
        """
        一键已读
        :param request:
        :return:
        """
        MessageCenterTargetUser.objects.filter(users=request.user).update(is_read=True)
        return DetailResponse(msg="设置成功")

    @action(methods=["POST"], detail=False, permission_classes=[IsAuthenticated])
    def resend_message(self, request):
        current_user = request.user
        # 默认按用户
        nid = request.data.get("nid", None)
        uid = request.data.get("uid", [])
        if not nid:
            return ErrorResponse(msg="参数错误")
        message_center_target_user_filter = Q()
        # message_center_target_user_filter.connector = 'OR'  # 默认为and
        new_message_center = MessageCenter.objects.filter(creator=current_user.id, id=nid).first()
        if len(uid):
            message_center_target_user_filter.children.append(("users__id__in", uid))
        u_info = (
            MessageCenterTargetUser.objects.filter(messagecenter=new_message_center)
            .filter(message_center_target_user_filter)
            .all()
        )

        return StreamingHttpResponse(
            websocket_to_message(u_info, new_message_center, current_user.username), content_type="text/event-stream"
        )

    @action(methods=["POST"], detail=False, permission_classes=[OpenApiPermission], authentication_classes=[])
    def open_api_create_message(self, request):
        """
        Open API
        :param request:
        :return: 第三方API发送通知
        """
        message_center = self.create(request)
        mid = message_center.data.get("data", {}).get("id", None)
        u_info = MessageCenterTargetUser.objects.filter(messagecenter__id=mid).all()
        websocket_to_message(u_info, message_center, "")
        return DetailResponse({"nid": mid}, "发送成功")

    @action(methods=["GET"], detail=False, permission_classes=[OpenApiPermission], authentication_classes=[])
    def open_api_get_message(self, request):
        agent_id = request.GET.get("agent_id", None)
        if not agent_id:
            return ErrorResponse("agent_id参数错误")
        queryset = MessageCenter.objects.filter(ext_kwargs__agent_id=agent_id).all()
        serializer = self.get_serializer(queryset, many=True, request=request)
        return SuccessResponse(data=serializer.data, msg="获取成功")


@csrf_exempt
def websocket_view(request):
    """
    websocket测试
    :param request:
    :return:
    """
    method = request.method.upper()
    if method == "GET":
        return render(request, "websocket.html")
    else:

        @action(methods=["POST"], detail=False, permission_classes=[])
        def send_ws_msg(self, request):
            from channels.layers import get_channel_layer

            channel_layer = get_channel_layer()
            from asgiref.sync import async_to_sync

            username = request.user.username.upper()
            nid = request.POST.get("nid", None)
            if nid is None:
                return DetailResponse({"code": 500, "msg": "缺少任务id"})
            for index in range(0, 20):
                obj = {"type": "chat_message", "message": str(datetime.now())}
                async_to_sync(channel_layer.group_send)(nid, obj)
            obj = {"type": "chat_message", "message": str(datetime.now())}
            async_to_sync(channel_layer.group_send)(username, obj)
            return DetailResponse()



# -*- coding: utf-8 -*-
"""测试用断言辅助。"""

from rest_framework import status


def assert_unauthorized(response):
    """
    断言未认证请求被拒绝：HTTP 401、404 或 HTTP 200 且 body 中 code != 2000（与项目统一错误响应一致）。
    """
    if response.status_code == status.HTTP_401_UNAUTHORIZED:
        return
    if response.status_code == status.HTTP_404_NOT_FOUND:
        return
    if response.status_code == 200:
        try:
            data = response.json()
            if isinstance(data, dict) and data.get("code") != 2000:
                return
        except Exception:
            pass
    raise AssertionError(
        f"Expected 401, 404 or 200 with code!=2000, got status={response.status_code} body={getattr(response, 'content', b'')[:300]!r}"
    )



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
@Project ：django_base_ai
@File    ：holidays.py
@Author  ：congxing.wang
@Date    ：2026/02/25
@Desc    ：节假日判断工具
"""

import logging
import os
import sys
from datetime import datetime, timedelta
from pathlib import Path

import django

logger = logging.getLogger(__name__)


class Holiday:
    def __init__(self):
        from django_base_ai import dispatch

        result = dispatch.get_dictionary_values("holidays")
        self.day_infos = {}
        children = result.get("children") or []
        logger.debug("节假日配置加载 %d 条", len(children))
        for item in children:
            label = item.get("label", "")
            raw = (item.get("value") or "").replace("to", "至").strip()
            if not raw:
                continue
            if "至" in raw and raw.count("至") >= 1:
                start_end = raw.split("至")
                if len(start_end) >= 2:
                    start_day = start_end[0].strip()
                    end_day = start_end[1].strip()
                    if start_day and end_day:
                        daterange_days = self.daterange(start_day, end_day)
                        self.day_infos.update({label: daterange_days})
                    continue
            if "," in raw:
                days = [x.strip() for x in raw.split(",") if x.strip()]
                if days:
                    self.day_infos.update({label: days})
                continue
            self.day_infos.update({label: [raw]})

    def daterange(self, start_day, end_day):
        dates = []
        dt = datetime.strptime(start_day, "%Y-%m-%d")
        dt_end = datetime.strptime(end_day, "%Y-%m-%d")
        while dt <= dt_end:
            t = datetime.strftime(dt, "%Y-%m-%d")
            dates.append(t)
            dt += timedelta(1)
        return dates

    def today(self, day=None):
        day = str(datetime.today().date()) if day is None or day == "" else day
        holiday_title = [k for k, v in self.day_infos.items() if day in v]
        return {
            "is_holiday": True if len(holiday_title) else False,
            "holiday_title": holiday_title[0] if len(holiday_title) else "",
            "day": day,
        }


if __name__ == "__main__":
    BASE_DIR = f"{Path(__file__).resolve().parent.parent}"
    sys.path.append(BASE_DIR)
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "DjangoBaseAi.settings")
    django.setup()

    h = Holiday()
    h.today()



# Generated by Django 5.2.11

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("system", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="ChunkUploadSession",
            fields=[
                ("id", models.BigAutoField(help_text="主键", primary_key=True, serialize=False, verbose_name="Id")),
                ("creator", models.CharField(blank=True, db_index=True, help_text="创建人", max_length=255, null=True, verbose_name="创建人")),
                ("modifier", models.CharField(blank=True, db_index=True, help_text="修改人", max_length=255, null=True, verbose_name="修改人")),
                ("dept_belong_id", models.CharField(blank=True, db_index=True, help_text="数据归属部门", max_length=255, null=True, verbose_name="数据归属部门")),
                ("update_datetime", models.DateTimeField(auto_now=True, db_index=True, help_text="修改时间", null=True, verbose_name="修改时间")),
                ("create_datetime", models.DateTimeField(auto_now_add=True, db_index=True, help_text="创建时间", null=True, verbose_name="创建时间")),
                ("description", models.TextField(blank=True, help_text="描述", null=True, verbose_name="描述")),
                ("upload_id", models.CharField(help_text="上传会话ID", max_length=64, unique=True, db_index=True, verbose_name="上传会话ID")),
                ("file_md5", models.CharField(help_text="文件MD5", max_length=32, db_index=True, verbose_name="文件MD5")),
                ("filename", models.CharField(blank=True, help_text="文件名", max_length=255, verbose_name="文件名")),
                ("total_size", models.BigIntegerField(default=0, help_text="文件总大小", verbose_name="文件总大小")),
                ("chunk_total", models.IntegerField(default=0, help_text="切片总数", verbose_name="切片总数")),
                (
                    "uploaded_chunk_indices",
                    models.JSONField(default=list, help_text="已上传的 chunk_index 列表，如 [0,1,2]", verbose_name="已上传切片序号"),
                ),
            ],
            options={
                "verbose_name": "切片上传会话",
                "verbose_name_plural": "切片上传会话",
                "db_table": "base_system_chunk_upload_session",
                "ordering": ("-create_datetime",),
            },
        ),
    ]



#!/usr/bin/env python
"""
@Project ：django_base_ai
@File    ：datetime_range_generator.py
@Author  ：congxing.wang
@Date    ：2026/02/25
@Desc    ：日期范围与月份列表生成
"""

from datetime import datetime, timedelta


def get_date_range(start_month=None, end_month=None):
    """
    获取日期范围及月份列表（不处理异常，调用方需自行捕获 ValueError）

    参数:
        start_month (str): 起始月份，格式为'YYYY-MM'（如 '2025-01'）
        end_month (str): 结束月份，格式为'YYYY-MM'（如 '2025-08'）

    返回:
        tuple: (start_date, end_date, months, start_month, end_month)
            - start_date: datetime 起始日期（精确到天）
            - end_date: datetime 结束日期（精确到天）
            - months: list[str] 月份列表（如 ['2025-01', '2025-02', ...]）
            - start_month: str 起始月份（如 '2025-01'）
            - end_month: str 结束月份（如 '2025-08'）

    异常:
        ValueError: 如果传入的 start_month 或 end_month 格式不正确

    示例:
        start_date, end_date, months, start_month, end_month = get_date_range('2025-01', '2025-08')
        或
        start_date, end_date, months, start_month, end_month = get_date_range()  # 默认最近12个月
    """
    if start_month and end_month:
        # 解析日期
        start_date = datetime.strptime(start_month, "%Y-%m")
        end_date = datetime.strptime(end_month, "%Y-%m") + timedelta(days=31)  # 确保包含整个月
        end_date = end_date.replace(day=1) - timedelta(days=1)  # 调整为月末
    else:
        # 默认最近12个月
        end_date = datetime.now()
        start_date = end_date - timedelta(days=365)
        start_month = start_date.strftime("%Y-%m")
        end_month = end_date.strftime("%Y-%m")

    # 生成月份范围
    months = []
    current = start_date
    while current <= end_date:
        months.append(current.strftime("%Y-%m"))
        # 下个月
        if current.month == 12:
            current = current.replace(year=current.year + 1, month=1)
        else:
            current = current.replace(month=current.month + 1)

    return start_date, end_date, months, start_month, end_month



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import logging
import traceback

from django.db.models import ProtectedError, RestrictedError
from django.http import Http404
from rest_framework.exceptions import APIException as DRFAPIException
from rest_framework.exceptions import AuthenticationFailed, PermissionDenied
from rest_framework.status import HTTP_401_UNAUTHORIZED
from rest_framework.views import exception_handler, set_rollback

from django_base_ai.utils.json_response import ErrorResponse

logger = logging.getLogger(__name__)


def custom_exception_handler(ex, context):
    """
    统一异常拦截处理
    目的:(1)取消所有的500异常响应,统一响应为标准错误返回
        (2)准确显示错误信息
    :param ex:
    :param context:
    :return:
    """
    msg = ""
    code = 4000
    # 调用默认的异常处理函数
    response = exception_handler(ex, context)
    if isinstance(ex, AuthenticationFailed):
        # 如果是身份验证错误
        detail = response.data.get("detail") if response else None
        if response and detail == "Given token not valid for any token type":
            code = 402
            msg = "身份认证已过期,请重新登录"
        elif response and detail == "Token is blacklisted":
            # token在黑名单
            return ErrorResponse(status=HTTP_401_UNAUTHORIZED)
        elif isinstance(detail, dict):
            # authentication.py 抛出的 detail={"code": 402, "msg": "..."}
            code = int(detail.get("code", 402))
            msg = detail.get("msg", "用户名或密码输入错误,请重新登录")
        else:
            code = 402
            msg = "用户名或密码输入错误,请重新登录"
    elif isinstance(ex, Http404):
        code = 400
        msg = "接口地址不正确"
    elif isinstance(ex, DRFAPIException):
        set_rollback()
        msg = ex.detail
        if isinstance(ex, PermissionDenied):
            msg = f"{msg} ({context['request'].method}: {context['request'].path})"
        if isinstance(msg, dict):
            for k, v in msg.items():
                for i in v:
                    msg = f"{k}:{i}"
    elif isinstance(ex, (ProtectedError, RestrictedError)):
        set_rollback()
        msg = "无法删除:该条数据与其他数据有相关绑定"
    # elif isinstance(ex, DatabaseError):
    #     set_rollback()
    #     msg = "接口服务器异常,请联系管理员"
    elif isinstance(ex, Exception):
        logger.exception(traceback.format_exc())
        msg = str(ex)
    return ErrorResponse(msg=msg, code=code)
